@@ -10,14 +10,14 @@
 !
 !	ARGUMENTS: see arguments in sglasso_ccm_path.f90
 subroutine sglasso_ccm_single(nSv,Sv,nTv,Tv_pkg,Tv_rw,nv,Tv_ptr,nTe,Te,nTe_ptr,Te_ptr,ne,Te_scn,Te_ptr_scn, &
-	nstep,tol,rho,grd,th,w,df,n,conv)
+	nstep,trnc,tol,rho,grd,th,w,n,conv)
 	integer	:: nSv,nTv,Tv_pkg(nTv),Tv_rw(nTv),nv,Tv_ptr(nv+1),nTe, Te(nTe),nTe_ptr,Te_ptr(nTe_ptr),ne, &
-		Te_scn(ne+1),Te_ptr_scn(ne+1),nstep,df,n,conv
-	double precision	:: grd(nv+ne),Sv(nSv),tol,rho,th(nv+ne),w(ne)
+		Te_scn(ne+1),Te_ptr_scn(ne+1),nstep,n,conv
+	double precision	:: grd(nv+ne),Sv(nSv),trnc,tol,rho,th(nv+ne),w(ne)
 	!internal variables
 	logical	:: A(ne),flg
 	integer	:: p,i,j,h,k,i0,i1,j0,j1,h0,h1,Te_scn_sz(ne), &
-		Te_cl_ptr(ne+1),Te_cl_sz(ne),Te_cl(ne*(Tv_ptr(nv+1)-1)),max_df
+		Te_cl_ptr(ne+1),Te_cl_sz(ne),Te_cl(ne*(Tv_ptr(nv+1)-1)),df,max_df
 	double precision	:: Svh(nSv),tr_SvTv(nv),tr_SvTe(ne),max_rho,Du,oth(nv+ne), &
 		dth,tr_SvhTeSvhTe,tr_SvhTvSvhTv,th_diff,Du_sgn
 	
@@ -62,7 +62,7 @@ subroutine sglasso_ccm_single(nSv,Sv,nTv,Tv_pkg,Tv_rw,nv,Tv_ptr,nTe,Te,nTe_ptr,T
 		Du = - tr_SvTe(i)
 		grd(nv + i) = Du
 		max_rho = max(max_rho,abs(Du) / w(i))
-		if(abs(Du).ge.rho) then
+		if(abs(Du) / w(i).ge.rho) then
 			A(i) = .true.
 			df = df + 1
 		end if
@@ -72,7 +72,6 @@ subroutine sglasso_ccm_single(nSv,Sv,nTv,Tv_pkg,Tv_rw,nv,Tv_ptr,nTe,Te,nTe_ptr,T
 	
 	n = 0
 	do
-	
 		oth(1:nv) = th(1:nv)
 		do i = 1, ne
 			if(A(i))	oth(nv + i) = th(nv + i)
@@ -98,22 +97,16 @@ subroutine sglasso_ccm_single(nSv,Sv,nTv,Tv_pkg,Tv_rw,nv,Tv_ptr,nTe,Te,nTe_ptr,T
 						grd(nv + i) = Du
 						Du_sgn = dsign(dble(1),Du)
 						Du = Du - rho * Du_sgn * w(i)
-						if(abs(Du).le.tol) exit
+                        if(abs(Du).le.tol) exit
 						call trSTeSTe(nSv,Svh,Te_scn_sz(i),Te(i0:i1),Tv_ptr(nv+1),Te_ptr(j0:j1), &
 							Te_cl_sz(i),Te_cl(h0:h1),tr_SvhTeSvhTe)
 						dth = Du / tr_SvhTeSvhTe
-						if(dsign(dble(1),th(nv + i) + dth).ne.Du_sgn) then
-							dth = - th(nv + i)
-							th(nv + i) = 0.
-							A(i) = .false.
-							df = df - 1
-							call updateSvh_e(p,nSv,Svh,dth,Te_scn_sz(i),Te(i0:i1),Tv_ptr(nv+1), &
-								Te_ptr(j0:j1))
-							exit
-						end if
+                        if(isnan(dth)) then
+                            conv = 2
+                            return
+                        end if
 						th(nv + i) = th(nv + i) + dth
-						call updateSvh_e(p,nSv,Svh,dth,Te_scn_sz(i),Te(i0:i1),Tv_ptr(nv+1), &
-							Te_ptr(j0:j1))
+						call updateSvh_e(p,nSv,Svh,dth,Te_scn_sz(i),Te(i0:i1),Tv_ptr(nv+1),Te_ptr(j0:j1))
 					end do
 				end if
 			end do
@@ -134,46 +127,59 @@ subroutine sglasso_ccm_single(nSv,Sv,nTv,Tv_pkg,Tv_rw,nv,Tv_ptr,nTe,Te,nTe_ptr,T
 				tr_SvhTvSvhTv = 0.
 				do h = i0 + 1, i1
 					do k = i0, h - 1
-						tr_SvhTvSvhTv = tr_SvhTvSvhTv + Svh(int(Tv_rw(k) + &
-							0.5 * Tv_rw(h) * (Tv_rw(h) - 1)))**2
+						tr_SvhTvSvhTv = tr_SvhTvSvhTv + Svh(int(Tv_rw(k) + Tv_rw(h) * (Tv_rw(h) - 1) / 2))**2
 					end do
 				end do
 				tr_SvhTvSvhTv = 2 * tr_SvhTvSvhTv + sum(Svh(Tv_pkg(i0:i1))**2)
 				dth = Du / tr_SvhTvSvhTv
+                if(isnan(dth)) then
+                    conv = 2
+                    return
+                end if
 				th(i) = th(i) + dth
 				call updateSvh_v(p,nSv,Svh,dth,Tv_ptr(i+1)-Tv_ptr(i),Tv_pkg(i0:i1),Tv_rw(i0:i1))
 			end do
 		end do
 		
-		th_diff = sum((th(1:nv) - oth(1:nv))**2)
-		if(th_diff.lt.tol) then
+		th_diff = sum(abs(th(1:nv) - oth(1:nv)))
+		if(th_diff/df.lt.tol) then
 			do i = 1, ne
 				if(A(i)) then
-					th_diff = th_diff + (th(nv + i) - oth(nv + i))**2 
+					th_diff = th_diff + abs(th(nv + i) - oth(nv + i))
 				end if
 			end do
 		end if
 		
-		if(th_diff.lt.tol) then
+		if(th_diff/df.lt.tol) then
 			flg = .true.
-			if(df.lt.max_df) then
-				do i = 1, ne
-					if(.not.A(i)) then
-						call trSTe(nSv,Svh,Te_scn_sz(i),Te(Te_scn(i):(Te_scn(i + 1) - 1)), &
-							Tv_ptr(nv+1),Te_ptr(Te_ptr_scn(i):(Te_ptr_scn(i + 1) - 1)),Du)
-						Du = Du - tr_SvTe(i)
-						grd(nv + i) = Du
-						if(abs(Du)/w(i).gt.rho) then
-							A(i) = .true.
-							df = df + 1
-							flg = .false.
-						end if
-					end if
-				end do
-			end if
+            do i = 1, ne
+                if(A(i).and.abs(th(nv + i)).gt.trnc) then
+                    if((th(nv + i) * grd(nv + i)).lt.0.) then
+                        flg = .false.
+                        call updateSvh_e(p,nSv,Svh,-th(nv + i),Te_scn_sz(i),Te(i0:i1),Tv_ptr(nv+1),Te_ptr(j0:j1))
+                        th(nv + i) = 0.
+                        A(i) = .false.
+                        df = df - 1
+                    end if
+                end if
+            end do
+            if(flg) then
+                do i = 1, ne
+                    if(.not.A(i)) then
+                        call trSTe(nSv,Svh,Te_scn_sz(i),Te(Te_scn(i):(Te_scn(i + 1) - 1)), Tv_ptr(nv+1), &
+                            Te_ptr(Te_ptr_scn(i):(Te_ptr_scn(i + 1) - 1)),Du)
+                        Du = Du - tr_SvTe(i)
+                        grd(nv + i) = Du
+                        if(abs(Du)/w(i) - rho .gt. 0.) then
+                            flg = .false.
+                            A(i) = .true.
+                            df = df + 1
+                        end if
+                    end if
+                end do
+            end if
 			if(flg) exit
 		end if
-			
 	end do
 	
 end subroutine sglasso_ccm_single
